"""
Django widgets for RemixIcon selection.
"""

import json
from django import forms
from django.urls import reverse
from django.utils.safestring import mark_safe

from django_remix_icon.remix_icons import REMIX_ICONS


def get_all_icon_names():
    """
    Get a flat list of all icon names with 'ri-' prefix.
    """
    icon_names = []
    for category, icons in REMIX_ICONS.items():
        for icon_name in icons.keys():
            icon_names.append(f"ri-{icon_name}")
    return icon_names


class IconSelectWidget(forms.Select):
    """
    A custom widget for selecting RemixIcons with autocomplete and preview functionality.

    This widget provides:
    - Autocomplete functionality for searching icons
    - Icon preview in the dropdown
    - Clean integration with Django admin
    """

    class Media:
        css = {
            'all': (
                'https://cdn.jsdelivr.net/npm/remixicon@4.7.0/fonts/remixicon.css',
                'django_remix_icon/css/icon_select.css',
            )
        }
        js = ('django_remix_icon/js/icon_select.js',)

    def __init__(self, attrs=None):
        super().__init__(attrs)
        self.choices = [(icon, icon) for icon in get_all_icon_names()]

    def format_value(self, value):
        """
        Return the value that should be displayed in the widget.
        """
        if value is None:
            return ''
        return str(value)

    def render(self, name, value, attrs=None, renderer=None):
        """
        Render the widget as HTML.
        """
        if attrs is None:
            attrs = {}

        field_id = attrs.get('id', f'id_{name}')

        preview_html = ''
        if value:
            preview_html = f'''
            <div class="icon-preview" id="icon-preview-{name}">
                <i class="{value}"></i>
                <span class="icon-name">{value}</span>
            </div>
            '''
        else:
            preview_html = f'''
            <div class="icon-preview" id="icon-preview-{name}" style="display: none;">
                <i class=""></i>
                <span class="icon-name"></span>
            </div>
            '''

        # Create a hidden input to store the actual value
        # This is what Django will use to save the data
        hidden_attrs = {
            'type': 'hidden',
            'name': name,
            'id': field_id,
            'value': value or '',
        }
        hidden_attrs_str = ' '.join([f'{k}="{v}"' for k, v in hidden_attrs.items()])

        # Get the search URL
        search_url = reverse('django_remix_icon:icon_search')

        # Combine HTML with autocomplete container
        html = f'''
        <div class="remix-icon-widget" data-field-name="{name}">
            <input {hidden_attrs_str}>
            <div class="icon-search-container">
                <input type="text" class="icon-search-input"
                       placeholder="Search RemixIcons..."
                       data-target="{field_id}"
                       data-icon-search-url="{search_url}"
                       autocomplete="off"
                       value="{value or ''}">
                <div class="icon-search-results" id="search-results-{name}"></div>
            </div>
            {preview_html}
        </div>
        '''

        return mark_safe(html)


class IconPreviewWidget(forms.TextInput):
    """
    A simpler widget that shows icon preview without autocomplete.
    Useful for read-only displays or simple forms.
    """

    class Media:
        css = {
            'all': (
                'https://cdn.jsdelivr.net/npm/remixicon@4.7.0/fonts/remixicon.css',
                'django_remix_icon/css/icon_preview.css',
            )
        }

    def render(self, name, value, attrs=None, renderer=None):
        """
        Render the widget with icon preview.
        """
        if attrs is None:
            attrs = {}

        attrs['class'] = attrs.get('class', '') + ' remix-icon-preview-input'

        input_html = super().render(name, value, attrs, renderer)

        preview_html = ''
        if value:
            preview_html = f'''
            <div class="icon-preview-simple">
                <i class="{value}"></i>
                <span class="icon-name">{value}</span>
            </div>
            '''

        html = f'''
        <div class="remix-icon-preview-widget">
            {input_html}
            {preview_html}
        </div>
        '''

        return mark_safe(html)



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

import os
import sys
import django

# Add the project root and app to Python path
sys.path.insert(0, os.path.abspath('..'))
sys.path.insert(0, os.path.abspath('../django_remix_icon'))

# Configure Django settings
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'docs.settings')
django.setup()

# -- Project information -----------------------------------------------------
project = 'Django RemixIcon'
copyright = '2025, Django RemixIcon Contributors'
author = 'Django RemixIcon Contributors'
release = '0.1.1'
version = '0.1.1'

# -- General configuration ---------------------------------------------------
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.napoleon',
    'sphinx.ext.viewcode',
    'sphinx.ext.intersphinx',
    'sphinx.ext.githubpages',
]

templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

language = 'en'

# -- Options for HTML output -------------------------------------------------
html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']

html_theme_options = {
    'canonical_url': '',
    'analytics_id': '',
    'logo_only': False,
    'display_version': True,
    'prev_next_buttons_location': 'bottom',
    'style_external_links': False,
    'vcs_pageview_mode': '',
    'style_nav_header_background': '#2980B9',
    'collapse_navigation': True,
    'sticky_navigation': True,
    'navigation_depth': 4,
    'includehidden': True,
    'titles_only': False,
}

html_context = {
    "display_github": True,
    "github_user": "brktrlw",
    "github_repo": "django-remix-icon",
    "github_version": "main",
    "conf_py_path": "/docs/",
}

html_title = f'{project} v{version}'
html_short_title = project

# -- Options for autodoc ----------------------------------------------------
autodoc_default_options = {
    'members': True,
    'member-order': 'bysource',
    'special-members': '__init__',
    'undoc-members': True,
    'exclude-members': '__weakref__'
}

# -- Options for intersphinx -------------------------------------------------
intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
    'django': ('https://docs.djangoproject.com/en/stable/', 'https://docs.djangoproject.com/en/stable/_objects/'),
}

# -- Options for napoleon (Google/NumPy docstring support) ------------------
napoleon_google_docstring = True
napoleon_numpy_docstring = True
napoleon_include_init_with_doc = False
napoleon_include_private_with_doc = False
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = False
napoleon_use_admonition_for_notes = False
napoleon_use_admonition_for_references = False
napoleon_use_ivar = False
napoleon_use_param = True
napoleon_use_rtype = True
napoleon_preprocess_types = False
napoleon_type_aliases = None
napoleon_attr_annotations = True

# -- Custom CSS and JS ------------------------------------------------------
html_css_files = [
    'custom.css',
]

# Add support for RemixIcon in docs
html_js_files = []



"""
Django app configuration for django-remix-icon.
"""

from django.apps import AppConfig


class DjangoRemixIconConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_remix_icon"
    verbose_name = "Django RemixIcon"



"""
URL configuration for documentation.
"""

from django.contrib import admin
from django.urls import path, include

urlpatterns = [
    path('admin/', admin.site.urls),
    path('remix-icon/', include('django_remix_icon.urls')),
]



"""
Django model fields for RemixIcon integration.
"""

from django import forms
from django.db import models
from django.core.exceptions import ValidationError

from django_remix_icon.remix_icons import REMIX_ICONS
from django_remix_icon.widgets import IconSelectWidget


def get_all_icon_names():
    """
    Get a flat list of all icon names with 'ri-' prefix.
    """
    icon_names = []
    for category, icons in REMIX_ICONS.items():
        for icon_name in icons.keys():
            icon_names.append(f"ri-{icon_name}")
    return icon_names


def get_icon_choices_list():
    """
    Get choices list for form field.
    """
    choices = []
    for category, icons in REMIX_ICONS.items():
        for icon_name in icons.keys():
            full_name = f"ri-{icon_name}"
            label = icon_name.replace('-', ' ').title()
            choices.append((full_name, label))
    return choices


class IconFormField(forms.ChoiceField):
    """
    Form field for selecting RemixIcons with autocomplete functionality.
    """

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('choices', get_icon_choices_list())
        kwargs.setdefault('widget', IconSelectWidget())
        super().__init__(*args, **kwargs)

    def validate(self, value):
        """
        Validate that the selected icon is a valid RemixIcon.
        """
        if value and value not in get_all_icon_names():
            raise ValidationError(f"'{value}' is not a valid RemixIcon name.")
        super().validate(value)


class IconField(models.CharField):
    """
    A Django model field for storing RemixIcon names.

    This field stores the icon name as a string and provides
    a custom form field with autocomplete functionality in Django admin.

    Usage:
        class MyModel(models.Model):
            icon = IconField()
    """

    description = "A field for storing RemixIcon names"

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_length', 100)
        kwargs.setdefault('blank', True)
        kwargs.setdefault('null', True)
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        """
        Return the appropriate form field for this model field.
        """
        defaults = {'form_class': forms.CharField}
        defaults.update(kwargs)

        field = super().formfield(**defaults)

        field.widget = IconSelectWidget()

        return field

    def validate(self, value, model_instance):
        """
        Validate that the stored value is a valid RemixIcon name.
        """
        super().validate(value, model_instance)
        if value and value not in get_all_icon_names():
            raise ValidationError(f"'{value}' is not a valid RemixIcon name.")

    def deconstruct(self):
        """
        Return details needed to recreate the field.
        """
        name, path, args, kwargs = super().deconstruct()
        if kwargs.get('max_length') == 100:
            del kwargs['max_length']
        return name, path, args, kwargs
